/*******************************************************************************

  Intel Data Center Bridging (DCB) Software
  Copyright(c) 2007-2008 Intel Corporation.

  This program is free software; you can redistribute it and/or modify it
  under the terms and conditions of the GNU General Public License,
  version 2, as published by the Free Software Foundation.

  This program is distributed in the hope it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
  more details.

  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc.,
  51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.

  The full GNU General Public License is included in this distribution in
  the file called "COPYING".

  Contact Information:
  e1000-eedc Mailing List <e1000-eedc@lists.sourceforge.net>
  Intel Corporation, 5200 N.E. Elam Young Parkway, Hillsboro, OR 97124-6497

*******************************************************************************/

#include <string.h>
#include "dcb_protocol.h"
#include "dcb_rule_chk.h"
#include "messages.h"

/******************************************************************************
 * This function checks DCB rules for DCBs settings.
 * The following rules are checked:
 * 1. The sum of bandwidth percentages of all Bandwidth Groups must total 100%.
 * 2. The sum of bandwidth percentages of all Traffic Classes within a Bandwidth
 *	Group must total 100.
 * 3. A Traffic Class should not be set to both Link Strict Priority
 *	and Group Strict Priority. ***** assumed to be already checked
 * 4. Link strict Bandwidth Groups can only have link strict traffic classes
 *      with zero bandwidth.
 * dcb_config - Struct containing DCB settings.
 * return : dcb_result
 *****************************************************************************/
dcb_result
dcb_check_config (pg_attribs *pg)
{
	u8 i, tx_bw, rx_bw, tx_bw_id, rx_bw_id;
	u8 tx_bw_sum[MAX_BW_GROUP],rx_bw_sum[MAX_BW_GROUP];
	boolean_t tx_link_strict[MAX_BW_GROUP], rx_link_strict[MAX_BW_GROUP];

	memset(tx_bw_sum,0,sizeof(tx_bw_sum));
	memset(rx_bw_sum,0,sizeof(rx_bw_sum));
	memset(tx_link_strict,0,sizeof(tx_link_strict));
	memset(rx_link_strict,0,sizeof(rx_link_strict));

	tx_bw = 0, rx_bw = 0;
	/* Check rules for Tx and Rx Bandwidth Groups */
	for (i = 0; i < MAX_BW_GROUP; i++) {
		tx_bw = tx_bw + pg->tx.bwg_percent_of_link[i];
	}
	if (tx_bw != BW_PERCENT) {
		/* only valid scenario for BWT!=100 is BWT==0 and all BWGs
		 * link strict
		 */
		for (i = 0; i < MAX_BW_GROUP; i++) {
			if ((tx_bw != 0) || (pg->tx.up[i].strict_priority !=
				dcb_link)) {
				log_message(MSG_ERR_DCB_INVALID_TX_TOTAL_BWG,
					"%d", (int)tx_bw);
				return dcb_bad_params;
			}
		}
	}
	for (i = 0; i < MAX_BW_GROUP; i++) {
		rx_bw = rx_bw + pg->rx.bwg_percent_of_link[i];
	}
	if (rx_bw != BW_PERCENT) {
		/* only valid scenario for BWT!=100 is BWT==0 and all BWGs
		 * link strict 
		 */
		for (i = 0; i < MAX_BW_GROUP; i++) {
			if ((rx_bw != 0) || (pg->rx.up[i].strict_priority !=
				dcb_link)) {
				log_message(MSG_ERR_DCB_INVALID_RX_TOTAL_BWG,
					"%d", (int)rx_bw);
				return dcb_bad_params;
			}
		}
	}

	/* Go through each traffic class and check rules for Tx and Rx */
	for (i = 0; i < MAX_TRAFFIC_CLASS; i++) {
		
		/* Since we assign strict priority to RX & TX via enumeration,
		 * from the data stores and from the peer. It would be
		 * impossible for both of them to be set.
		 * So that is no longer checked in this function.
		 */

		/* Transmit Check */ 
		tx_bw = 0, tx_bw_id = 0;
		tx_bw = (u8)(pg->tx.up[i].percent_of_bandwidth_group_cap);
		tx_bw_id = (u8)(pg->tx.up[i].bandwidth_group_mapping);

		if (tx_bw_id >= MAX_BW_GROUP) {
			log_message(MSG_ERR_DCB_INVALID_TX_BWG_IDX, "%d",
				(int)tx_bw_id);
			return  dcb_bad_params;
		}
		if (pg->tx.up[i].strict_priority == dcb_link) {
			tx_link_strict[tx_bw_id] = TRUE;
			/* Link strict should have zero bandwidth */
			if (tx_bw){
				log_message(MSG_ERR_DCB_INVALID_TX_LSP_NZERO_BW_TC,
					"%d%d", i, (int)tx_bw);
				return dcb_bad_params;
			}
		} else if (!tx_bw) {
			log_message(MSG_ERR_DCB_INVALID_TX_ZERO_BW_TC, "%d", i);
			/* Non link strict should have non zero bandwidth*/
			return dcb_bad_params;
		}
		/* Receive Check */
		rx_bw = 0, rx_bw_id = 0;
		rx_bw = (u8)(pg->rx.up[i].percent_of_bandwidth_group_cap);
		rx_bw_id = (u8)(pg->rx.up[i].bandwidth_group_mapping);

		if (rx_bw_id >= MAX_BW_GROUP) {
			log_message(MSG_ERR_DCB_INVALID_RX_BWG_IDX, "%d",
				(int)rx_bw_id);
			return dcb_bad_params;
		}	   
		if (pg->rx.up[i].strict_priority == dcb_link) {
			rx_link_strict[rx_bw_id] = TRUE;
			/* Link strict class should have zero bandwidth */
			if (rx_bw){
				log_message(MSG_ERR_DCB_INVALID_RX_LSP_NZERO_BW_TC,
					"%d%d", i, (int)rx_bw);
				return dcb_bad_params;
			}
		} else if (!rx_bw) {
			log_message(MSG_ERR_DCB_INVALID_RX_ZERO_BW_TC, "%d", i);
			/* Non link strict class should have non-zero bw */
			return dcb_bad_params; /* DCB_RX_ERR_TC_BW_ZERO; */
		}
		tx_bw_sum[tx_bw_id] = tx_bw_sum[tx_bw_id] + tx_bw;
		rx_bw_sum[rx_bw_id] = rx_bw_sum[rx_bw_id] + rx_bw;

	}

	/* Transmit Check */
	for (i = 0; i < MAX_BW_GROUP; i++) {
		/* sum of bandwidth percentages of all traffic classes within
		 * a Bandwidth Group must total 100 except for link strict
		 * group (zero bandwidth).
		 */
		if (tx_link_strict[i]) {
			if (tx_bw_sum[i]) {
				log_message(MSG_ERR_DCB_INVALID_TX_LSP_NZERO_BWG,
					"%d%d", i, (int)tx_bw_sum[i]);
				/* Link strict group should have zero bw */
				return dcb_bad_params;
			}
		} else if (tx_bw_sum[i] != BW_PERCENT && tx_bw_sum[i] != 0) {
			log_message(MSG_ERR_DCB_INVALID_TX_BWG, "%d%d",
					i, (int)tx_bw_sum[i]);
			return dcb_bad_params;
		}
	}
	/* Receive Check */
	for (i = 0; i < MAX_BW_GROUP; i++) {
		/* sum of bandwidth percentages of all traffic classes
		 * within a Bandwidth Group must total 100 except for
		 * link strict group ( zero bandwidth).
		 */
		if (rx_link_strict[i]) {
			if (rx_bw_sum[i]) {
				log_message(MSG_ERR_DCB_INVALID_RX_LSP_NZERO_BWG,
					"%d%d", i, (int)rx_bw_sum[i]);
				/* Link strict group should have zero bw */
				return dcb_bad_params;
			}
		} else if (rx_bw_sum[i] != BW_PERCENT && rx_bw_sum[i] != 0) {
			log_message(MSG_ERR_DCB_INVALID_RX_BWG, "%d%d",
					i, (int)rx_bw_sum[i]);
			return dcb_bad_params;
		}
	}
	return dcb_success;
}
